WintergraspFu = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "FuBarPlugin-2.0", "AceEvent-2.0")
WintergraspFu:RegisterDB("WintergraspFuDB")
WintergraspFu:RegisterDefaults('profile', {
		showestimate = true
	}
)


local options = {
	type = "group",
	args = {
		showestimate = {
			type = 'toggle',
			name = "Show estimate",
			desc = "Show best estimate when exact time is not available.",
			get = function() return WintergraspFu.db.profile.showestimate end,
			set = function() WintergraspFu.db.profile.showestimate = not WintergraspFu.db.profile.showestimate end,
		}
	}
}

WintergraspFu.OnMenuRequest = options
WintergraspFu.defaultPosition = "LEFT"



function WintergraspFu:OnInitialize()
end
   
function WintergraspFu:AddonMessage(arg1, arg2, arg3, arg4)
	if arg1 ~= "WgFu" then
		return
	end
--[===[@alpha@
	ChatFrame1:AddMessage(arg1 .. arg2 .. arg3 .. arg4)
--@end-alpha@]===]
	local posdiff = string.match(arg2, "%+(%d*)$")
	if posdiff ~= nil then
		self.db.realm.nextbattle = time() + posdiff;
	end
	if arg2 == "?" and self.db.realm.nextbattle ~= nil and self.db.realm.nextbattle > time() then
		SendAddonMessage("WgFu", "+" .. self.db.realm.nextbattle - time(), "WHISPER", arg4);
	end
end

function WintergraspFu:UpdateState()
	self:CheckZone()
end

local worldstatetext = nil
local worldstatetime = nil
function WintergraspFu:CheckZone()
	if GetZoneText() ~= "Wintergrasp" then
		return
	end
	local uiType, state, text = GetWorldStateUIInfo(8)
	worldstatetext = text
	worldstatetime = time()
end

local lasttickworldstate = nil
local lastusedworldstate = nil
function WintergraspFu:DelayedCheckWorldState()
	if worldstatetext ~= nil and lasttickworldstate == worldstatetext and lastusedworldstate ~= worldstatetext then
		-- World state constant between two ticks and not same as last used; update!
		local h, m, s = string.match(worldstatetext, "Next Battle: (%d):(%d%d):(%d%d)$")
		if s ~= nil then 
			local newnextbattle = worldstatetime + h*3600 + m*60 + s;
			self:SetNextBattle(newnextbattle, 2);
		end
		lastusedworldstate = worldstatetext
	end
	lasttickworldstate = worldstatetext
end


function WintergraspFu:SyncNextBattle(newnextbattle)
--	ChatFrame1:AddMessage("SyncNextBattle(" .. newnextbattle .. ")")
	SendAddonMessage("WgFu", "+" .. newnextbattle - time(), "GUILD");
end

function WintergraspFu:SetNextBattle(newnextbattle, accuracy)
	if self.db.realm.nextbattle == nil or math.abs(newnextbattle - self.db.realm.nextbattle) > accuracy then
		self:SyncNextBattle(newnextbattle);
	end
	self.db.realm.nextbattle = newnextbattle;
end

function WintergraspFu:EverySecond()
	self:UpdateText()
	self:DelayedCheckWorldState()
end

function WintergraspFu:OnEnable()
	self:RegisterEvent("UPDATE_WORLD_STATES", "UpdateState")
	self:RegisterEvent("CHAT_MSG_ADDON", "AddonMessage")
	self:RegisterEvent("GOSSIP_SHOW", "GossipShow")
	self:ScheduleRepeatingEvent(self.EverySecond, 1, self)
	if self.db.realm.nextbattle == nil or self.db.realm.nextbattle < time() - 600 then
		SendAddonMessage("WgFu", "?", "GUILD");
	end
end

function WintergraspFu:GossipShow()
	local text = GetGossipText()
	local h, m, s = string.match(text, "The next battle begins in (%d):(%d*):(%d*)%.");
	if s ~= nil then 
		local newnextbattle = time() + h*3600 + m*60 + s;
		self:SetNextBattle(newnextbattle, 10);
	end
end

function WintergraspFu:OnTextUpdate()
	if self:IsTextColored() then
		self:SetText("|cffffffffWG:|r |cffffff00" .. self:CreateStatusText() .. "|r")
	else
		self:SetText("WG: " .. self:CreateStatusText())
	end
end

function WintergraspFu:GetTimeText(delta)
	if delta < 0 then 
		return "Active"
	end
	local h = math.floor(delta / 3600)
	local m = math.floor(delta % 3600 / 60)
	local s = delta % 60
	local text = ""
	if h > 0 then
		text = text .. h .. "h "
	end
	if m > 0 then
		text = text .. m .. "m "
	end
	text = text .. s .. "s"
	return text
end

function WintergraspFu:GetTimeText2(delta)
	if delta < 0 then 
		return "Active"
	end
	local h = math.floor(delta / 60)
	local m = math.floor(delta % 60)
	local text = ""
	if h > 0 then
		text = text .. h .. "h "
	end
	text = text .. m .. "m "
	return text
end

function WintergraspFu:CreateStatusText()
	if self.db.realm.nextbattle == nil then
		return "Unknown"
	end
	local diff = self.db.realm.nextbattle - time()
	if diff >= 0 then
		return self:GetTimeText(diff)
	end
	if not self.db.profile.showestimate then
		return "Unknown"
	end
	diff = math.floor(-diff/60)
	-- Estimating more than a few battles away is meaningless
	if diff > 180*4 then 
		return "Unknown"
	end
	local longestimate = 150 - (diff + 150) % 180 -- estimate for full length battles
	local shortestimate = 150 - (diff + 150) % 165 -- estimate for short battles
	return "~" .. self:GetTimeText2(shortestimate) .. " .. " .. self:GetTimeText2(longestimate)
end

function WintergraspFu:OnDisable()

end




